/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 * 
 * Contributions from:
 *     Ryan Wagoner <ryan@wgnrs.dynu.com>, 2006
 *          
 * Released under the GPL v2. */

#include "imspector.h"

#define PLUGIN_NAME "Bad-words IMSpector content plugin"
#define PLUGIN_SHORT_NAME "Bad-words"

#define DEFAULT_REPLACE_CHAR '*'

extern "C"
{
	bool initcontentplugin(struct contentplugininfo &contentplugininfo,
		class Options &options, bool debugmode);
	void closecontentplugin(void);
	int contentmanipulator(char *originalbuffer, char *modifiedbuffer);
};

int readbadwords(std::string filename);

std::vector<std::string> badwords;
char replacecharacter = DEFAULT_REPLACE_CHAR;
bool repalceall = false;
bool localdebugmode = false;

bool initcontentplugin(struct contentplugininfo &contentplugininfo,
	class Options &options, bool debugmode)
{
	if (options["badwords_filename"] == "") return false;

	localdebugmode = debugmode;
	
	int count = readbadwords(options["badwords_filename"]);

	if (count == -1)
	{
		syslog(LOG_ERR, PLUGIN_SHORT_NAME ": Couldn't open bad words file %s",
			options["badwords_filename"].c_str());
		return false;
	}
	
	if (options["badwords_replace_character"] != "")
		replacecharacter = options["badwords_replace_character"][0];
		
	syslog(LOG_INFO, PLUGIN_SHORT_NAME ": Loaded %d bad-words, replacing with '%c'",
		count, replacecharacter);

	contentplugininfo.pluginname = PLUGIN_NAME;
	
	return true;
}

void closecontentplugin(void)
{
	return;
}

/* The main plugin function. See contentplugin.cpp. */
int contentmanipulator(char *originalbuffer, char *modifiedbuffer)
{
	debugprint(localdebugmode, PLUGIN_SHORT_NAME ": filtering before: original: %s modified: %s",
		originalbuffer, modifiedbuffer);

	for (std::vector<std::string>::iterator i = badwords.begin();
		i != badwords.end(); i++)
	{
		const char *needle = (*i).c_str();
		size_t needlelength = (*i).length();
		
		char *s = modifiedbuffer;
		
		while ((s = strcasestr(s, needle)))
		{
			if (s > modifiedbuffer)
			{
				if (isalpha(*(s - 1)) && isalpha(*(s + needlelength)))
				{
					s++;
					continue;
				}
			}
		
			memset(s, (int) replacecharacter, needlelength);
		}
	}
	
	debugprint(localdebugmode, PLUGIN_SHORT_NAME ": filtering after: modified: %s", modifiedbuffer);
	
	return 0;
}

int readbadwords(std::string filename)
{
	std::ifstream input(filename.c_str());
	char buffer[STRING_SIZE];
	int result = 0;
	
	memset(buffer, 0, STRING_SIZE);
	
	if (!input) return -1;
	
	while (input)
	{	
		if (!input.getline(buffer, STRING_SIZE)) break;
			
		stripnewline(buffer);
		
		if (!strlen(buffer)) break;
		
		badwords.push_back(buffer); result++;
	}

	return result;
}
