/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2. */

#include "imspector.h"

#define PLUGIN_NAME "Misc IMSpector filter plugin"
#define PLUGIN_SHORT_NAME "Misc"

extern "C"
{
	bool initfilterplugin(struct filterplugininfo &filterplugininfo,
		class Options &options, bool debugmode);
	void closefilterplugin(void);
	bool eventfilter(std::vector<struct imevent> &imevents);
};

bool blockfiles = false;
bool blockwebcams = false;
bool localdebugmode = false;

bool initfilterplugin(struct filterplugininfo &filterplugininfo,
	class Options &options, bool debugmode)
{
	localdebugmode = debugmode;

	filterplugininfo.pluginname = PLUGIN_NAME;
		
	if (options["block_files"] == "on")
	{
		blockfiles = true;
		debugprint(localdebugmode, PLUGIN_SHORT_NAME ": Blocking all file transfers");
	}
	if (options["block_webcams"] == "on")
	{
		blockwebcams = true;
		debugprint(localdebugmode, PLUGIN_SHORT_NAME ": Blocking all webcams");
	}

	return true;
}

void closefilterplugin(void)
{
	return;
}

/* The main plugin function. See eventplugin.cpp. */
bool eventfilter(std::vector<struct imevent> &imevents)
{
	bool lastfiltered = false;

	for (std::vector<struct imevent>::iterator i = imevents.begin();
		i != imevents.end(); i++)
	{
		if ((*i).type == TYPE_FILE)
		{
			if (blockfiles)
			{
				(*i).filtered = true;
				lastfiltered = true;
				continue;
			}
		}
		if ((*i).type == TYPE_WEBCAM)
		{
			if (blockwebcams)
			{
				(*i).filtered = true;
				lastfiltered = true;
				continue;
			}
		}
	}

	debugprint(localdebugmode, PLUGIN_SHORT_NAME ": Filtering: %d", lastfiltered);
	
	return lastfiltered;
}
