/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2. */

struct protocolplugininfo
{
	std::string pluginname;
	std::string protocolname;
	uint16_t port; 				/* Network byte order. */
};

#define TYPE_NULL 0
#define TYPE_MSG 1
#define TYPE_FILE 2
#define TYPE_TYPING 3
#define TYPE_WEBCAM 4

struct imevent
{
	time_t timestamp;
	std::string clientaddress;
	std::string protocolname;	/* From the protocol plugin. */
	bool outgoing;
	int type;
	std::string localid;
	std::string remoteid;
	bool filtered;
	std::string eventdata;
};

struct messageextent
{
	int start;
	int length;
};

typedef bool (*initprotocolplugintype)(struct protocolplugininfo &protocolplugininfo,
	class Options &options, bool debugmode);
typedef void (*closeprotocolplugintype)(void);
typedef int (*processpackettype)(bool outgoing, class Socket &incomingsock, char *replybuffer, 
	int *replybufferlength, std::vector<struct imevent> &imevents, std::string &clientaddress,
	std::vector<struct messageextent> &messageextents);

class ProtocolPlugin
{
	public:
		struct protocolplugininfo protocolplugininfo;
	
		ProtocolPlugin();
		~ProtocolPlugin();
		bool loadplugin(std::string filename);
		bool unloadplugin(void);
		
		bool callinitprotocolplugin(class Options &options, bool debugmode);
		void callcloseprotocolplugin(void);
		int callprocesspacket(bool outgoing, class Socket &incomingsock, char *replybuffer,
			int *replybufferlength, std::vector<struct imevent> &imevents, std::string &clientaddress,
			std::vector<struct messageextent> &messageextents);

	private:
		void *handle;

		initprotocolplugintype initprotocolplugin;
		closeprotocolplugintype closeprotocolplugin;
		processpackettype processpacket;
};
